﻿unit uFbInformationEXE;

// |===========================================================================|
// | UNITE uFbInformationEXE                                                   |
// | F.BASSO 2013                                                              |
// |___________________________________________________________________________|
// | Unité contenant la class tfbInformationEXE permettant d'avoir les         |
// | informations de versions d'une application ou d'une DLL                   |
// | par défaut a la création il y chargement de la première traduction trouvée|
// |___________________________________________________________________________|
// | Ce programme est libre, vous pouvez le redistribuer et ou le modifier     |
// | selon les termes de la Licence Publique Générale GNU publiée par la       |
// | Free Software Foundation .                                                |
// | Ce programme est distribué car potentiellement utile,                     |
// | mais SANS AUCUNE GARANTIE, ni explicite ni implicite,                     |
// | y compris les garanties de commercialisation ou d'adaptation              |
// | dans un but spécifique.                                                   |
// | Reportez-vous à la Licence Publique Générale GNU pour plus de détails.    |
// |                                                                           |
// | anbasso@wanadoo.fr                                                        |
// |___________________________________________________________________________|
// | Versions                                                                  |
// |  _________                                                                |
// | /  1.0.0  \______________________________________________________________ |
// || 31/07/2013 F.BASSO                                                      ||
// || Création de l'unité                                                     ||
// ||_________________________________________________________________________||
// |===========================================================================|

interface
uses classes;
type
  TfbInformationEXE = class (TObject)
  private
    fpVersionInfo        : pointer;
    fbooVersionInfo      : boolean;
    fstrComments         : string;
    fstrInternalName     : string;
    fstrProductName      : string;
    fstrCompanyName      : string;
    fstrLegalCopyright   : string;
    fstrProductVersion   : string;
    fstrFileDescription  : string;
    fstrLegalTrademarks  : string;
    fstrPrivateBuild     : string;
    fstrFileVersion      : string;
    fstrOriginalFilename : string;
    fstrSpecialBuild     : string;
    flstTranslation      : tstringlist;
    procedure chargerListetraduction ;
  public
    constructor create (strNomFichier : string);
    destructor Destroy ; override;
    property strComments         : string read fstrComments ;
    property strInternalName     : string read fstrInternalName ;
    property strProductName      : string read fstrProductName ;
    property strCompanyName      : string read fstrCompanyName ;
    property strLegalCopyright   : string read fstrLegalCopyright ;
    property strProductVersion   : string read fstrProductVersion ;
    property strFileDescription  : string read fstrFileDescription ;
    property strLegalTrademarks  : string read fstrLegalTrademarks ;
    property strPrivateBuild     : string read fstrPrivateBuild ;
    property strFileVersion      : string read fstrFileVersion ;
    property strOriginalFilename : string read fstrOriginalFilename ;
    property strSpecialBuild     : string read fstrSpecialBuild ;
    property lstTranslation      : TStringList read flstTranslation ;
    procedure chargerInformation (strLangCodepage : string );
    function  getStrInformation ( strquery : string ) : string;
    procedure getlstStrInformation ( lststrquery : tstrings );
  end;
implementation
uses
{$IF CompilerVersion >= 22}
  system.SysUtils ,winapi.windows;
{$ELSE}
  sysutils,windows;
{$IFEND}

type
  TTransRec = packed record
    Lang,
    CharSet: Word;
  end;
  PTransRec = ^TTransRec;

// #===========================================================================#
// #== TfbInformationEXE ======================================================#
// #                                                                           #
// # Partie privée                                                             #
// #                                                                           #
// #===========================================================================#
// #===========================================================================#

procedure TfbInformationEXE.chargerListetraduction;

//  ___________________________________________________________________________
// | Procedure TfbInformationEXE.chargerListetranslation                       |
// | _________________________________________________________________________ |
// || Permet de charger la liste des traductions                              ||
// ||_________________________________________________________________________||
// || Entrées |                                                               ||
// ||         |                                                               ||
// ||_________|_______________________________________________________________||
// || Sorties |                                                               ||
// ||         |                                                               ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

var
  translation : PTransRec;
  intTaille : DWORD;
  intRecCount : Integer;
  indexTranslation : Integer;
  intlang , intCharset : word;
begin
  flstTranslation.Clear;

// ****************************************************************************
// *  Chargement de la liste des traductions                                  *
// ****************************************************************************

  VerQueryValue( fpVersionInfo , PChar('\VarFileInfo\Translation'), Pointer(translation), intTaille );

// ****************************************************************************
// *  Calcule du nombre de traduction trouvée                                 *
// ****************************************************************************

  intRecCount := intTaille div SizeOf(TTransRec);

// ****************************************************************************
// * Conversion en hexadécimal et stockage dans la liste                      *
// ****************************************************************************

  for indexTranslation := 0 to intRecCount -1 do
  begin
    intlang := translation^.Lang;
    intCharset := translation^.CharSet;
    flstTranslation.Add(Format('%4.4x%4.4x',[intlang,intcharset] ));
    Inc(translation);
  end;
end;

// #===========================================================================#
// #== TfbInformationEXE ======================================================#
// #                                                                           #
// # Partie publique                                                           #
// #                                                                           #
// #===========================================================================#
// #===========================================================================#

constructor TfbInformationEXE.create (strNomFichier : string);

//  ___________________________________________________________________________
// | constructor TfbInformationEXE.create                                      |
// | _________________________________________________________________________ |
// || Initialise les variables lors de la création                            ||
// ||_________________________________________________________________________||
// || Entrées |  strNomFichier : string                                       ||
// ||         |    Nom complet du fichier                                     ||
// ||_________|_______________________________________________________________||
// || Sorties |                                                               ||
// ||         |                                                               ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

var
  intTaille : dword;
begin
  inherited create;
  flstTranslation := tstringlist.Create ;

// ****************************************************************************
// *  Demande la taille des informations                                      *
// ****************************************************************************

  intTaille := GetFileVersionInfoSize(PChar(strNomFichier ), intTaille);
  fbooVersionInfo := intTaille > 0;
  If fbooVersionInfo Then
  begin

// ****************************************************************************
// *  Réservation en mémoire d'une zone de la taille voulue                   *
// ****************************************************************************

    fpVersionInfo := AllocMem(intTaille);

// ****************************************************************************
// *  Copie dans le buffer des informations                                   *
// ****************************************************************************

    GetFileVersionInfo(PChar(strNomFichier), 0, intTaille, fpVersionInfo );
    chargerListetraduction;
    if flstTranslation.Count > 0  then
      chargerInformation(flstTranslation[0]);
  end;
end;

destructor TfbInformationEXE.Destroy;

//  ___________________________________________________________________________
// | destructor TfbInformationEXE.Destroy                                      |
// | _________________________________________________________________________ |
// || Permet de liberrer le ressources alouées                                ||
// ||_________________________________________________________________________||
// || Entrées |                                                               ||
// ||         |                                                               ||
// ||_________|_______________________________________________________________||
// || Sorties |                                                               ||
// ||         |                                                               ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

begin
  flstTranslation.Free ;
  freemem(fpVersionInfo);
  inherited destroy;
end;

procedure TfbInformationEXE.chargerInformation (strLangCodepage : string);

//  ___________________________________________________________________________
// | Procedure TfbInformationEXE.chargerInformation                            |
// | _________________________________________________________________________ |
// || Permet de charger les informations relative à une langue et un code page||
// ||_________________________________________________________________________||
// || Entrées | strLangCodepage : string                                      ||
// ||         |   code hexa correpondant à la langue et au code page          ||
// ||         |   040C04E4 pour la France                                     ||
// ||_________|_______________________________________________________________||
// || Sorties |                                                               ||
// ||         |                                                               ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

Var
  Taille      : DWord;
  strresultat : PChar;
begin
  if fbooVersionInfo then
  begin
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\FileVersion'), Pointer(strresultat), Taille) Then
      fstrFileVersion := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\Comments'), Pointer(strresultat), Taille) Then
      fstrComments := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\InternalName'), Pointer(strresultat), Taille) Then
      fstrInternalName := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\ProductName'), Pointer(strresultat), Taille) Then
      fstrProductName := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\CompanyName'), Pointer(strresultat), Taille) Then
      fstrCompanyName := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\LegalCopyright'), Pointer(strresultat), Taille) Then
      fstrLegalCopyright := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\ProductVersion'), Pointer(strresultat), Taille) Then
      fstrProductVersion := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\FileDescription'), Pointer(strresultat), Taille) Then
      fstrFileDescription := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\LegalTrademarks'), Pointer(strresultat), Taille) Then
      fstrLegalTrademarks := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\PrivateBuild'), Pointer(strresultat), Taille) Then
      fstrPrivateBuild := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\OriginalFilename'), Pointer(strresultat), Taille) Then
      fstrOriginalFilename := strresultat ;
    If VerQueryValue(fpVersionInfo, PChar('\StringFileInfo\' + strLangCodepage + '\SpecialBuild'), Pointer(strresultat), Taille) Then
      fstrSpecialBuild := strresultat ;
  End;
end;

function TfbInformationEXE.getStrInformation ( strquery : string ) : string;

//  ___________________________________________________________________________
// | Procedure TfbInformationEXE.getStrInformation                             |
// | _________________________________________________________________________ |
// || Permet de retourner les informations dans une chaine en fonction des    ||
// || variables                                                               ||
// ||_________________________________________________________________________||
// || Entrées | strquery : string                                             ||
// ||         |   les variables sont les suivantes                            ||
// ||         |   %Comments%                                                  ||
// ||         |   %InternalName%                                              ||
// ||         |   %ProductName%                                               ||
// ||         |   %CompanyName%                                               ||
// ||         |   %LegalCopyright%                                            ||
// ||         |   %ProductVersion%                                            ||
// ||         |   %FileDescription%                                           ||
// ||         |   %LegalTrademarks%                                           ||
// ||         |   %PrivateBuild%                                              ||
// ||         |   %FileVersion%                                               ||
// ||         |   %OriginalFilename%                                          ||
// ||         |   %SpecialBuild%                                              ||
// ||_________|_______________________________________________________________||
// || Sorties | Result : string                                               ||
// ||         |   chaine d'information                                        ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

begin
  result := StringReplace (strquery,'%Comments%' ,fstrComments,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%InternalName%' ,fstrInternalName ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%ProductName%' ,fstrProductName ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%CompanyName%' ,fstrCompanyName ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%LegalCopyright%' ,fstrLegalCopyright ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%ProductVersion%' ,fstrProductVersion ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%FileDescription%' ,fstrFileDescription ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%LegalTrademarks%' ,fstrLegalTrademarks,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%PrivateBuild%' ,fstrPrivateBuild ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%FileVersion%' ,fstrFileVersion ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%OriginalFilename%' ,fstrOriginalFilename ,[rfReplaceAll ,rfIgnoreCase ]);
  result := StringReplace (result,'%SpecialBuild%' ,fstrSpecialBuild ,[rfReplaceAll ,rfIgnoreCase ]);
end;

procedure TfbInformationEXE.getlstStrInformation (lstStrquery : tstrings );

//  ___________________________________________________________________________
// | procedure TfbInformationEXE.getlstStrInformation                          |
// | _________________________________________________________________________ |
// || Permet de retourner les informations dans une liste de chaine en        ||
// || fonctions des variables                                                 ||
// ||_________________________________________________________________________||
// || Entrées | lstStrquery : tstringlist                                     ||
// ||         |   liste de chaines a modifier                                 ||
// ||         |   les variables sont les suivantes                            ||
// ||         |   %Comments%                                                  ||
// ||         |   %InternalName%                                              ||
// ||         |   %ProductName%                                               ||
// ||         |   %CompanyName%                                               ||
// ||         |   %LegalCopyright%                                            ||
// ||         |   %ProductVersion%                                            ||
// ||         |   %FileDescription%                                           ||
// ||         |   %LegalTrademarks%                                           ||
// ||         |   %PrivateBuild%                                              ||
// ||         |   %FileVersion%                                               ||
// ||         |   %OriginalFilename%                                          ||
// ||         |   %SpecialBuild%                                              ||
// ||_________|_______________________________________________________________||
// || Sorties |                                                               ||
// ||         |                                                               ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

var
  indexLigne : integer;
begin
  for indexLigne  := 0 to lststrquery.Count-1 do
  begin
    lststrquery[indexLigne] := getStrInformation(lststrquery[indexLigne]) ;
  end;
end;

end.


unit Unit1;
{Microsoft Speech API lit les chiffres en français par Caribensila}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TForm1 = class(TForm)
    rbt_Anglais: TRadioButton;
    rbt_Francais: TRadioButton;
    Edit1: TEdit;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
  private
    { Déclarations privées }
  public
    { Déclarations publiques }
  end;

var
  Form1: TForm1;

implementation

uses ComObj;//Pour la fonction "CreateOLEObject"

{$R *.dfm}

var
  Vox: variant;

const
  Unite:array[1..19] of String=(' ain',' deu',' troa',' catre',' cinq',' sees',
  ' sett',' huit',' nef',' dis',' honze',' douze',' treize',' catorze',
  ' qinze',' seieze',' di sett',' dis huit',' dis nef');
  Dizaine:array[2..9]of String=(' veint',' trant',' kaarant',' seincant',
  ' soissantt',' soissantt',' catre veint',' catre veint');
  Cardinal:array[0..3]of String=(' zaant',' mil,',' million,',' milliar,');

procedure TForm1.FormCreate(Sender: TObject);
begin
  Vox:=CreateOLEObject('SAPI.SpVoice');//Crée un objet unique non initialisé de la classe spécifiée par le paramètre 'SAPI.SpVoice'
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
Vox:=UnAssigned;//Désassigne une valeur vierge à la variable de type variant
end;

procedure Dis(N:Double); //Extended); treal
var
  PE,PF,P : Int64;
  Temp,Phrase : string;
  NbreZero,P1,P10,P100,Rang,i,j : Byte;
  Neg : Boolean;
begin
  Neg := false;
  if N<=0 then begin
    Neg:=true;
    N:=-N;
  end;
  if (N>999999999999.999) then exit;
  //On sépare le réel en partie entière et partie fractionnaire
  PF:=0;
  if pos(DecimalSeparator,Floattostr(N))<>0 then begin//si réel...
    Temp := copy(Floattostr(N),pos(DecimalSeparator,Floattostr(N))+1,20);
    NbreZero:=0;
    repeat//Pour les cas comme "3.0001", par exemple
      if Pos('0',Temp)=1 then begin
        Inc(NbreZero);
        delete(Temp,1,1);
      end;
    until not (Pos('0',Temp)=1);
    PF := StrToInt(Temp);
  end;
  PE := Trunc(N);//Partie entière
  Phrase:='.';
  For i := 1 to 2 do begin
    if i=1 then P:=PF ;
    if i=2 then begin
      if PE=0 then Phrase:= ' zarro'+Phrase;
      P:=PE;
    end;
    Rang:=0;
    if ((PF<>0)and(i=1)) or ((PE<>0)and(i=2)) then begin
      repeat
        P1:=P mod 10;//Isoler les unités
        P:=P div 10;
        P10:=P mod 10;//Isoler les dizaines
        P:=P div 10;
        if P10 in [1,7,9] then Inc(P1,10);
        if P1>0 then Phrase:=Unite[P1]+Phrase;
        if P10>1 then begin
          if (P10<8) and ((P1=1) or (P1=11)) then Phrase:=' é'+Phrase;
          Phrase:=Dizaine[P10]+Phrase;
        end;
        P100:=P mod 10;//Isoler les centaines
        P:=P div 10;
        if (P100>0)and(P100<10) then begin
          Phrase:=' cent'+Phrase;
          if P100>1 then Phrase:=Unite[P100]+Phrase;
        end;
        if P>0 then begin
          Inc(Rang);
          if P mod 1000>0 then Phrase:=Cardinal[Rang]+Phrase;
          if (P mod 1000=1) and (Rang=1) then Dec(P);
        end;
      until P=0;
      if i=1 then begin
        for j:= 1 to NbreZero do Phrase:=' zarro,'+Phrase;
        Phrase:=', virgu''l,'+Phrase;
      end;
    end;{if}
  end;{for}
  if (Neg) then Vox.Speak('négatif,'+Phrase) else Vox.Speak(Phrase);
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
  if (rbt_Anglais.Checked)  then Vox.Speak(Edit1.Text);;
  if (rbt_Francais.Checked) then Dis(StrToFloat(Edit1.Text));
end;

procedure TForm1.Edit1KeyPress(Sender: TObject; var Key: Char);
  //Filtre de l'Edit
begin
if not (Key in['0'..'9','-',#13,Chr(VK_BACK),Chr(VK_DELETE),Chr(VK_SEPARATOR)])
then Key:=#9;
end;

end.
